@@ -1,8 +1,12 @@
 /*
- * commands.c - Mail protocol independent command handlers.
+vim:ts=2:sw=2:
+RJVB: cleaned up the tab mess...
+ * commands.c -- POP3 command handlers
+ * hotwayd is a POP3-HTTPMail gateway. It will allow you to
+ * check your HOTMAIL account from your unix box.
  *
  * Created by: Espeleta Tomas <espeleta@libero.it>, 12-Apr-2001
- * POP3 code based on nupop-0.4, by Kevin Stone <kstone@nuvox.net> 
+ * POP3 code based on nupop-0.4, by Kevin Stone <kstone@nuvox.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,57 +26,68 @@
 
 #include "libghttp-1.0.9-mod/ghttp.h"
 #include "libghttp-1.0.9-mod/ghttp_constants.h"
-#include "hotwayd.h"
 #include "httpmail.h"
+#include "hotwayd.h"
 #include "inet.h"
 #include <sys/time.h>
-#include "commands.h" 
-#include "xmlstuff.h"
 
 /* Global Vars */
-URL_STRUCT *trash = NULL; /* where we store the trash href */
-URL_STRUCT *smtpurl = NULL; /* where we store the smtpurl href */
-static KEYPAIR **cookies = NULL;
+KEYPAIR **cookies = NULL;
+FOLDER_STRUCT *inbox_props = NULL;
 ghttp_request *request = NULL;
 time_t time_getlist; /* time when getlist() was last called, used to update LIST */
 #define GETLIST_TIMEOUT 120 /* 2 minute timeout for list command to refresh */
 const char *badpwdtxt[] = { "Bad Password", "Invalid login." };
 const int badpwdcnt = sizeof(badpwdtxt) / sizeof(char *);
-extern char *prog;
 
+int lazy_dump(char *pos, int len, int *curpos_p, int *curline_p, int maxlines);
+static void prepare_and_send(ghttp_request *request, char *url);
+static int set_authinfo(ghttp_request *request, char *UserName, char *Password,
+			char *WWWAuthenticate, int maintainlogged);
+static int clear_authinfo(ghttp_request *request);
+
+
+extern int cur_message;
+extern int top_lines;  /* how many lines to display using top */
+extern int pastheader; /* used for displaying the message using top */
 char buffer[N_BUFFER];
 char *serverurl=NULL;
 char xml_length[N_XML_LENGTH];
 char *UserName = NULL;  /* allocate space depending on size later */
-char *PassWord = NULL;
+char PassWord[N_PASSWORD];
 char *WWWAuthenticate;
-int unauth_cnt = 0; /* check for unauth loop on proxy */
+static int unauth_cnt = 0; /* check for unauth loop on proxy */
+
 
 /* proxy variables are setup via command line arguments */
-static char *proxy=NULL;  /* proxy address:port */
-static char *proxy_username=NULL;
-static char *proxy_password=NULL;
-static char *access_list=NULL;
+char *proxy=NULL;  /* proxy address:port */
+char *proxy_username=NULL;
+char *proxy_password=NULL;
 extern char *folder;
 
-
-/* Some function prototypes */
-static int clear_authinfo(ghttp_request *request);
-static int check_cpmv_response(int reply);
-static int proppatch(char *href, char *xml_request);
-static int allowed_user(char *user);
+/* Extern stuff */
+extern char *dir_names_request; /* request for folder hrefs */
+extern char *dir_props_request; /* request for folder properties */
+extern char *msg_props_request; /* request for messages within folder props */
+int ParseXMLMsgList(ghttp_request *request, FOLDER_STRUCT *folder_props);
+int ParseXMLFolderProps(ghttp_request *request, char *foldername, FOLDER_STRUCT *folder_props);
+char *ParseXMLFolderRoot(ghttp_request *request, FOLDER_STRUCT *folder_props);
 
 
 /* general counters/flags */
 int oldstatus      = 0;
 int cookiecount    = 0;
 int auth_required  = 0;
+int listdownloaded = 0; /* got the inbox list yet? */
 extern int log_level;
 
+typedef struct HTTP_Servers{
+	char *domain;
+	char *URL;
+} HTTP_Servers;
 
 const HTTP_Servers default_server[]= {
 	{ "hotmail.com", "http://oe.hotmail.com/cgi-bin/hmdata" },
-	{ "hotmail.co.uk", "http://oe.hotmail.com/cgi-bin/hmdata" },
 	{ "msn.com", "http://oe.msn.msnmail.hotmail.com/cgi-bin/hmdata" },
 	{ "lycos.co.uk", "http://webdav.lycos.co.uk/httpmail.asp" },
 	{ "lycos.ch", "http://webdav.lycos.de/httpmail.asp" },
@@ -82,10 +97,18 @@ const HTTP_Servers default_server[]= {
 	{ "lycos.at", "http://webdav.lycos.at/httpmail.asp" },
 	{ "lycos.nl", "http://webdav.lycos.nl/httpmail.asp" },
 	{ "spray.se", "http://webdav.spray.se/httpmail.asp" },
-	{ "caramail.com", "http://webdav.caramail.lycos.fr/httpmail.asp" },
 };
 
 
+/* Defintions for the types of http authorisation methods */
+#define UNKNOWN 0
+#define BASIC 1
+#define DIGEST 2
+
+typedef struct Auth_Method {
+	char *wwwauthstr;
+	int auth_type;
+} Auth_Method;
 
 const Auth_Method auth_method[] = {
   { "Basic", BASIC },
@@ -115,7 +138,7 @@ const char *http_hdr_Allow_Rename = "Allow-Rename";
 /* ---------------------------------------------------------------------
  * SET Cookies, taking them from array of structures and verifing domain
  */
-static void my_set_cookies(ghttp_request *request, char *url)
+void my_set_cookies(ghttp_request *request, char *url)
 {
 	int i;
 	char mybuf[N_MYBUF];
@@ -138,7 +161,7 @@ static void my_set_cookies(ghttp_request *request, char *url)
 /* ---------------------------------------------------------------------
  * Parse and save all cookies passed by response
  */
-static void save_cookies(ghttp_request *request)
+void save_cookies(ghttp_request *request)
 {
 	int i, j = -1, n;
 	char *k, *key, *value, *cookiedomain = NULL;
@@ -167,9 +190,8 @@ static void save_cookies(ghttp_request *request)
 		if (value[0] == '\"') value++;
 
 		if (!strcmp(key, "domain")) { /* Read domain for this cookie */
-		        cookiedomain = strdup(value); 
-			/* the next time causes problems with httpmail_destroy so removed */
-			/* if (j != -1) cookies[j]->domain = cookiedomain; */
+			cookiedomain = strdup(value);
+			if (j != -1) cookies[j]->domain = cookiedomain;
 		}
 		else if (strcmp(key, "path")) { /* Read key/value for this cookie, ignoring path*/
 
@@ -198,7 +220,7 @@ static void save_cookies(ghttp_request *request)
 int propfind(ghttp_request *request, char *myurl, int whichrequest)
 {
 	int status;
-
+	
 	/* Settings of our connection	 */
 	ghttp_clean(request);
 	ghttp_set_uri(request, myurl);
@@ -209,19 +231,19 @@ int propfind(ghttp_request *request, char *myurl, int whichrequest)
 	/* Set Body request & Depth header */
 	switch(whichrequest) {
 	case MSGFOLDERROOT:
-	  if (ghttp_set_body(request, (char *)dir_names_request, (int)(strlen(dir_names_request)) ) == -1)
+	  if (ghttp_set_body(request, dir_names_request, (int)(strlen(dir_names_request)) ) == -1)
 		return -1;
 	  ghttp_set_header(request, http_hdr_Depth, "0");
 	  snprintf(xml_length, N_XML_LENGTH, "%d", (int)(strlen(dir_names_request)));
 	  break;
-	case GETFOLDERPROPS:
-	  if (ghttp_set_body(request, (char *)dir_props_request, (int)(strlen(dir_props_request)) ) == -1)
+	case INBOXPROPS:
+	  if (ghttp_set_body(request, dir_props_request, (int)(strlen(dir_props_request)) ) == -1)
 		return -1;
 	  ghttp_set_header(request, http_hdr_Depth, "1, noroot");
 	  snprintf(xml_length, N_XML_LENGTH, "%d", (int)(strlen(dir_props_request)));
 	  break;
 	case MSGLIST:
-	  if (ghttp_set_body(request, (char *)msg_props_request, (int)(strlen(msg_props_request)) ) == -1)
+	  if (ghttp_set_body(request, msg_props_request, (int)(strlen(msg_props_request)) ) == -1)
 		return -1;
 	  ghttp_set_header(request, http_hdr_Depth, "1, noroot");
 	  snprintf(xml_length, N_XML_LENGTH, "%d", (int)(strlen(msg_props_request)));
@@ -250,9 +272,7 @@ int propfind(ghttp_request *request, char *myurl, int whichrequest)
 	  set_authinfo(request, UserName, PassWord, WWWAuthenticate, 0);
 
 	}
-
-	/* TODO: Check if we have return number 51 which is password requried before nulling out auth */
-	if (whichrequest == MSGLIST && !auth_required)
+	if (whichrequest == MSGLIST)
 	  set_authinfo(request, UserName, PassWord, NULL, 1);
 
 	/* now prepare and send our http request that we've built */
@@ -329,7 +349,7 @@ int handlereplies(ghttp_request *request, int reply)
 
 
 /* use this function to fill in the URL if we get moved response */
-char *get_moved_url(ghttp_request *req) {
+static char *get_moved_url(ghttp_request *req) {
   char *new_url=NULL;
   
   if (ghttp_get_header(request, http_hdr_Location) != NULL) {
@@ -343,9 +363,9 @@ char *get_moved_url(ghttp_request *req) {
 }
 
 
-int set_authinfo(ghttp_request *request, char *UserName, char *Password,
+static int set_authinfo(ghttp_request *request, char *UserName, char *Password,
 			char *WWWAuthenticate, int maintainlogged) {
-  int ret = -1;
+  int ret;
   clear_authinfo(request);
   switch(auth_name) {
   case DIGEST:
@@ -379,370 +399,194 @@ static int clear_authinfo(ghttp_request *request) {
  */
 int dologin(ghttp_request *request) 
 {
-  int reply = 0;
-  char *root_href = NULL;
-  
-  /* A cycle of propfind requests, until MULTISTATUS is returned */
-  while ((reply = propfind(request, serverurl, MSGFOLDERROOT)) != MULTISTATUS) {
-    if (reply == MOVED) {
-      free(serverurl);
-      serverurl = get_moved_url(request);
+    int reply = 0;
+    char *root_href = NULL;
+
+    /* A cycle of propfind requests, until MULTISTATUS is returned */
+    while ((reply = propfind(request, serverurl, MSGFOLDERROOT)) != MULTISTATUS) {
+      if (reply == MOVED) {
+	free(serverurl);
+	serverurl = get_moved_url(request);
+      }
+      if (handlereplies(request, reply)) 
+	break;
     }
-    if (handlereplies(request, reply)) 
-      break;
-  }
-  /* So here we have either logged in or bombed out */
-  unauth_cnt=0; /* reset unauth counter */
+    /* So here we have either logged in or bombed out */
+    unauth_cnt=0; /* reset unauth counter */
 
-  /* lets try to find the root href of the mailbox so we can call dir_props */
-  if (reply == MULTISTATUS &&	(root_href = ParseXMLFolderRoot(request, trash, smtpurl)) != NULL) {
-    if (serverurl) free(serverurl);
-    serverurl = strdup(root_href);  /* copy folder root location */
-    if (root_href) free(root_href);
+    /* lets try to find the root href of the mailbox so we can call dir_props */
+    if (reply == MULTISTATUS &&	(root_href = ParseXMLFolderRoot(request, inbox_props)) != NULL) {
+      if (serverurl) free(serverurl);
+      serverurl = strdup(root_href);  /* copy folder root location */
+      if (root_href) free(root_href);
 
-    return 1;
-  }
+      /* So at this point we are logged into hotmail, but now let's see if we
+       * can get into the folder that we have requested */
+      /* A cycle of propfind requests, until MULTISTATUS is returned */
+      while ((reply = propfind(request, serverurl, INBOXPROPS)) != MULTISTATUS) {
+	if (reply == MOVED) {
+	  free(serverurl);
+	  serverurl = get_moved_url(request);
+	}
+        if (handlereplies(request, reply))
+	  break;
+      }
+      unauth_cnt=0; /* reset unauth counter */
 
-  /* otherwise return the status reply which caused login to fail */
-  return reply;
-}
-
-/* Function: request_folder_props()
- * Send a HTTP request for the mailbox folder properties.
- */
-void request_folder_props(ghttp_request *request)
-{
-  int reply = 0;
-  while ((reply = propfind(request, serverurl, GETFOLDERPROPS)) != MULTISTATUS) {
-    if (reply == MOVED) {
-      free(serverurl);
-      serverurl = get_moved_url(request);
+      /* Lets parse the XML and see if we can find the folder we were after */
+      return ParseXMLFolderProps(request, folder, inbox_props);
     }
-    if (handlereplies(request, reply))
-      break;
-  }
-  unauth_cnt=0; /* reset unauth counter */
-  return;
+
+    /* otherwise return our bombed out reply */
+    return reply;
 }
 
 /* ---------------------------------------------------------------------
- * Get List of messages from server for specified folder
+ * Get List of messages from server
  */
-int getlist(ghttp_request *request, FOLDER_STRUCT *folder_props)
+int getlist(ghttp_request *request)
 {
 	int reply = 0, downloaded = 0;
 
-	if (!folder_props || !folder_props->href)
-	  return downloaded;
-	
 	time(&time_getlist);
 	
 	/* A cycle of propfind requests, until MULTISTATUS is returned */
-	while ((reply = propfind(request, folder_props->href, MSGLIST)) != MULTISTATUS) {
+	while ((reply = propfind(request, inbox_props->href, MSGLIST)) != MULTISTATUS) {
 	  if (reply == MOVED) {
-		free(folder_props->href);
-		folder_props->href = get_moved_url(request);
+		free(inbox_props->href);
+		inbox_props->href = get_moved_url(request);
 	  }
 	  if (handlereplies(request, reply))
 		break;
 	}
-	
-	if (reply == MULTISTATUS && ParseXMLMsgList(request, folder_props))
+	if (reply == MULTISTATUS && ParseXMLMsgList(request, inbox_props))
 		downloaded = 1;
 	unauth_cnt=0; /* reset unauth counter */
 	return downloaded;
 }
 
-
-/** Function cp_mv_msg()<br>
- * This function performs the copying or moving of messages between folders on
- * the HTTPMail server. Possible values for cmd are COPY, MOVE or TRASH. If
- * TRASH is given as cmd then the dest_href is ignore and we use the global
- * trash->href address. Copying and moving are essentially the same operation
- * just that we send a COPY or MOVE respectively in the HTTP request. We can
- * set this using the libghttp functions.
+/* ---------------------------------------------------------------------
+ * Get messages from server
  */
-int cp_mv_msg(ghttp_request *request, int num_msg, FOLDER_STRUCT *folder_props, char *dest_href, int cmd)
+char *getmessage(ghttp_request *request, int num_msg)
 {
-  char *msg_name;
-  int reply=0;
-  
-  if (!folder_props || !folder_props->list_props || !folder_props->list_props[num_msg]->href )
-    return E_CPMV_NOFOLDERPROPS;
 
-  ghttp_clean(request); /* clean up the request structure */
-  
-  /* get the file name of the message */
-  msg_name = strrchr(folder_props->list_props[num_msg]->href, '/');
-  msg_name++;
+	if (inbox_props->list_props ){
+		if( inbox_props->list_props[num_msg]->message ){
+		  int curpos = 0;
+		  int curline = 0;
+		  PSOUT("+OK We have that cached\r\n");
+		  lazy_dump(inbox_props->list_props[num_msg]->message, -1, &curpos, &curline, top_lines);
+		  PSOUT(".\r\n");
+		  return( inbox_props->list_props[num_msg]->message );
+		}
+		else if( inbox_props->list_props[num_msg]->href ) {
+			/* Settings of our connection	 */
+			ghttp_clean(request);
+			ghttp_set_uri(request, inbox_props->list_props[num_msg]->href);
+			ghttp_set_type(request, ghttp_type_lazy);
 
-  /* Now we will put the destination href into the buffer and following that
-   * append to file name of the message. At the same time we will set the
-   * type of operation (either copy or move) depending on the value cmd. */
-  switch(cmd) {
-  case TRASH:
-    if (!trash || !trash->href)
-      return E_MOVE_TRASH_NOT_SET;
-    strlcpy(buffer, trash->href, N_BUFFER);
-    ghttp_set_type(request, ghttp_type_move);
-    break;
-  case MOVE:
-    if (!dest_href)
-      return E_MOVE_NO_DEST;
-    strlcpy(buffer, dest_href, N_BUFFER);
-    ghttp_set_type(request, ghttp_type_move);
-    break;
-  case COPY:
-    if (!dest_href)
-      return E_COPY_NO_DEST;
-    strlcpy(buffer, dest_href, N_BUFFER);
-    ghttp_set_type(request, ghttp_type_copy);
-    break;
-  default:
-    return E_UNKNOWN_ERR;
-  }
+			/* Set Headers */
+			ghttp_set_header(request, http_hdr_Accept, "message/rfc822, */*");
+			ghttp_set_header(request, http_hdr_Translate, "f");
+			set_authinfo(request, UserName, PassWord, NULL, 1);
 
-  /* append the message name to the buffer */
-  strlcat(buffer, msg_name, N_BUFFER);
-    
-  /* Setup the rest of the connection. Send request to message href, set
-   * HTTP destination header to where we want to move/copy msg to. */
-  ghttp_set_uri(request, folder_props->list_props[num_msg]->href);
-  ghttp_set_header(request, http_hdr_Destination, buffer);
-  ghttp_set_header(request, http_hdr_Allow_Rename, "t");
-  ghttp_set_header(request, http_hdr_Content_Length, "0");
-  set_authinfo(request, UserName, PassWord, NULL, 1);
-  prepare_and_send(request, folder_props->list_props[num_msg]->href);
-		
-  /* is response ok? (should be 201) */
-  reply=ghttp_status_code(request);
+			/* Prepare and process the connection */
+			prepare_and_send(request, inbox_props->msg_props[num_msg]->href);
 
-  return check_cpmv_response(reply);
+			cur_message = num_msg;
+
+			return NULL;
+		}
+	}
+	return NULL;
 }
 
-/** Function: delhref()<br>
- * Essentially the same as cp_mv_msg just that it takes an already formed href
- * and msg_name. This is useful if the folder_structs variable is not filled
- * in (basically if we need to cleanup after a failed copy).
+/* ---------------------------------------------------------------------
+ * Delete messages from server (moving it to hm trash)
  */
-int delhref(ghttp_request *request, char *to_delete, char *msg_name) {
-  int reply;
-  
-  ghttp_clean(request);
-  if (!trash || !trash->href)
-    return E_MOVE_TRASH_NOT_SET;
+int delmessage(ghttp_request *request, int num_msg)
+{
+	char *msg_name;
 
-  strlcpy(buffer, trash->href, N_BUFFER);
-  strlcat(buffer, msg_name, N_BUFFER);
-  ghttp_set_type(request, ghttp_type_move);
-  ghttp_set_uri(request, to_delete);
-  ghttp_set_header(request, http_hdr_Destination, buffer);
-  ghttp_set_header(request, http_hdr_Allow_Rename, "t");
-  ghttp_set_header(request, http_hdr_Content_Length, "0");
-  set_authinfo(request, UserName, PassWord, NULL, 1);
-  prepare_and_send(request, to_delete);
+	if (!inbox_props->trash)
+		return 0;
+
+	if (inbox_props->list_props && inbox_props->list_props[num_msg]->href ) {
+		/* Set message destination (trash) */
+		msg_name = strrchr(inbox_props->list_props[num_msg]->href, '/');
+		msg_name++;
+		strlcpy(buffer, inbox_props->trash, N_BUFFER);
+		strlcat(buffer, msg_name, N_BUFFER);
+			
+		/* Settings of our connection	 */
+		ghttp_clean(request);
+		ghttp_set_uri(request, inbox_props->list_props[num_msg]->href);
+		ghttp_set_type(request, ghttp_type_move);
+
+		/* Set Headers */
+		ghttp_set_header(request, http_hdr_Destination, buffer);
+		ghttp_set_header(request, http_hdr_Allow_Rename, "t");
+		ghttp_set_header(request, http_hdr_Content_Length, "0");
+		set_authinfo(request, UserName, PassWord, NULL, 1);
+
+		prepare_and_send(request, inbox_props->list_props[num_msg]->href);
 		
-  /* is response ok? (should be 201) */
-  reply=ghttp_status_code(request);
-
-  return check_cpmv_response(reply);
-}
-
-/* This function takes as inputs the MSG_STRUCT and the desired read state and
- * updates the read flag on the HTTPMail server. It does this by building a
- * D:propertyupdate XML request using the href from the MSG_STRUCT. It then
- * calls the proppatch function to wrap the HTTP request around the XML
- * instruction. */
-int update_readmark(MSG_STRUCT *msg_props, int new_readmark) {
-  char *xml_request;
-  int reply;
-  
-  if (!msg_props)
-    return E_NOMSGPROPS;
-
-  if (msg_props->readmark == new_readmark)
-    return E_OK; /* no point updating if it is already the same */
-
-  switch(new_readmark) {
-  case 0:
-    xml_request = build_propertyupdate_str("0");
-    break;
-  case 1:
-    xml_request = build_propertyupdate_str("1");
-    break;
-  default:
-    return E_INVALID_READMARK;
-  }
-
-  while ((reply = proppatch(msg_props->href, xml_request)) != OKSTATUS) {
-    if (reply == MULTISTATUS)
-      break;
-    if (reply == MOVED) {
-      free(msg_props->href);
-      msg_props->href = get_moved_url(request);
-    }
-    if (handlereplies(request, reply))
-      break;
-  }
-
-  free(xml_request);
-
-  return (reply == OKSTATUS || reply == MULTISTATUS) ? E_OK : E_UPDATE_FAILED;
-}
-
-void setnewsmtpurl(void) {
-    if (smtpurl->href) {
-	free(smtpurl->href);
-	smtpurl->href = NULL;
-    }
-    smtpurl->href = get_moved_url(request);
-}
-
-int handlesmtpreplies(int reply) {
-    const char *wwwauth;
-
-    if (reply != AUTHORIZATION)
-	unauth_cnt=0; /* reset the unauthorised counter */
-    if (reply == AUTHORIZATION) {
-	if ((wwwauth = ghttp_get_header(request, http_hdr_WWW_Authenticate)) != NULL) {
-	    if (WWWAuthenticate)
-		free(WWWAuthenticate);
-	    WWWAuthenticate = strdup(wwwauth);
-	    auth_required = 1;
-#ifdef DEBUG
-	    fprintf( stderr, "reply=AUTHORIZATION; %s\n", wwwauth );
-#endif
+		/* is response ok? (should be 201) */
+		if (ghttp_status_code(request) >= 200 && ghttp_status_code(request) < 300)
+		  return 1;
+		else
+		  return 0;
 	}
 	return 0;
-    }
-    if (reply == MOVED)
-	return 0;
-    return reply;
 }
 
-/* handling message posting, mainly for hotsmtpd */
-int post(char *body, unsigned long body_len) {
-    int status, ret_len;
-
-    ghttp_clean(request);
-    ghttp_set_uri(request, smtpurl->href);
-    if (ghttp_set_type(request, ghttp_type_post) == -1) 
-	return E_GHTTP_ERR;
-    if (ghttp_set_body(request, body, body_len) == -1)
-	return E_GHTTP_ERR;
-    /* Do not go wildly adding headers.  httpmail servers are very picky *
-     * about what headers they see.  If this function stops working properly *
-     * someone went screwing around with headers here or in prepare_and_send. *
-     * If this isn't it, then httpmail has changed its interface! */  
-    ghttp_set_header(request, "Content-Type", "message/rfc821");
-    set_authinfo(request, UserName, PassWord, NULL, 1);
-
-    prepare_and_send(request,smtpurl->href);
-
-    status = ghttp_status_code(request);
-    save_cookies(request);
-
-    if (status == AUTHORIZATION) {
-	int i;
-	if (unauth_cnt++ > 3) /* make it 4 times just to be sure :-) */
-	    return PASSWORDFAILED;
-	for (i = 0; i < badpwdcnt; i++) {
-	    if (!strcmp(badpwdtxt[i], ghttp_reason_phrase(request))) 
-		return PASSWORDFAILED;
-	}
-    } else
-	auth_required = 0;
-
-    return status;
-}
-
-static int proppatch(char *href, char *xml_request) {
-  int status, len = strlen(xml_request);
-
-  ghttp_clean(request);
-
-  ghttp_set_uri(request, href);
-
-  if (ghttp_set_type(request, ghttp_type_proppatch) == -1)
-    return E_GHTTP_ERR;
-
-  if (ghttp_set_body(request, xml_request, len) == -1) {
-    free(xml_request);
-    return E_GHTTP_ERR; 
-  }
-
-  ghttp_set_header(request, http_hdr_Content_Type, "text/xml");
-  ghttp_set_header(request, http_hdr_Brief, "t");
-  snprintf(xml_length, N_XML_LENGTH, "%d", (int)(strlen(xml_request)));
-  ghttp_set_header(request, http_hdr_Content_Length, xml_length);
-  set_authinfo(request, UserName, PassWord, NULL, 1);
-  prepare_and_send(request, href);
-  save_cookies(request);
-
-  if ((status = ghttp_status_code(request)) == AUTHORIZATION) {
-    int i;
-    if (unauth_cnt++ > 3) /* make it 4 times just to be sure :-) */
-      return PASSWORDFAILED;
-    for (i = 0; i < badpwdcnt; i++) {
-      if (!strcmp(badpwdtxt[i], ghttp_reason_phrase(request))) 
-	return PASSWORDFAILED;
-    }
-  } else
-    auth_required = 0;
-  
-  return status;
-}
-
-/** Function check_cpmv_response()
- * Checks the status code returned from the last request and returns an error
- * message according to what happened.
- */
-static int check_cpmv_response(int reply) {
-  if (reply >= 200 && reply <= 201)
-    return E_CPMV_OK;
-  else if (reply >= 202 && reply < 300)
-    return E_CPMV_WEIRD_OK;
-  else if (reply == 403)
-    return E_CPMV_FORBIDDEN;
-  else if (reply == 409)
-    return E_CPMV_CONFLICT;
-  else if (reply == 412)
-    return E_CPMV_PRECOND_FAILED;
-  else if (reply == 423)
-    return E_CPMV_LOCKED;
-  else if (reply == 503)
-    return E_CPMV_BADGATEWAY;
-  else if (reply == 507)
-    return E_CPMV_INSUFFICIENT_STORAGE;
-  else
-    return E_UNKNOWN_ERR;
-}
-
-
 /* ---------------------------------------------------------------------
  * Functions called by hotwayd.c. They are a translation POP3 -> HTTPMail
  * ---------------------------------------------------------------------
  */
 
+long httpmail_nemails(void)
+{
+	return inbox_props->listcount;
+}
 
 void httpmail_destroy(void)
 {
+	int i;
+
+	/* Free memory */
+	if (inbox_props != NULL) {
+		free(inbox_props->href);
+		free(inbox_props->trash);
+		free(inbox_props->name);
+		if (inbox_props->visiblecount > 0 && listdownloaded) {
+			for (i = 0; i < inbox_props->visiblecount; i++) {
+				if( inbox_props->msg_props[i]->href ){
+					free(inbox_props->msg_props[i]->href);
+				}
+				if( inbox_props->msg_props[i]->message ){
+					free(inbox_props->msg_props[i]->message);
+				}
+				free(inbox_props->msg_props[i]);
+			}
+		}
+		listdownloaded=0; /* 20020924: reset listdownloaded variable */
+		free(inbox_props->msg_props);
+		inbox_props->msg_props= NULL;
+		free(inbox_props->unread_props);
+		inbox_props->unread_props=NULL;
+		free(inbox_props);
+	}
+	inbox_props= NULL;
+
 	/* ensure that we reset cookiecount back to 0! */
 	while(cookiecount>0) {
 		cookiecount--;
 		if( cookies[cookiecount] ){
-			if ( cookies[cookiecount]->key ) { 
-				free(cookies[cookiecount]->key);
-				cookies[cookiecount]->key = NULL;
-			}
-			if ( cookies[cookiecount]->value ) {
-				free(cookies[cookiecount]->value);
-				cookies[cookiecount]->value = NULL;
-			}
-			if ( cookies[cookiecount]->domain ) {
-				free(cookies[cookiecount]->domain);
-				cookies[cookiecount]->domain = NULL;
-			}
+			free(cookies[cookiecount]->key);
+			free(cookies[cookiecount]->value);
+			free(cookies[cookiecount]->domain);
 			free(cookies[cookiecount]);
 			cookies[cookiecount]= NULL;
 		}
@@ -750,26 +594,12 @@ void httpmail_destroy(void)
 	
 	if (UserName)  free(UserName);
 	UserName=NULL;
-	if (PassWord)  free(PassWord);
-	PassWord=NULL;
 	if (serverurl) free(serverurl);
 	serverurl=NULL;
 	if (cookies)   free(cookies);
 	cookies= NULL;
 	if (request)   ghttp_request_destroy(request);
-	request = NULL;
-	if (trash && trash->href) {
-	  free(trash->href);
-	  trash->href=NULL;
-	}
-	if (trash) free(trash);
-	trash=NULL;
-	if (smtpurl && smtpurl->href) {
-	  free(smtpurl->href);
-	  smtpurl->href=NULL;
-	}
-	if (smtpurl) free(smtpurl);
-	smtpurl=NULL;
+
 }
 
 /* ---------------------------------------------------------------------
@@ -778,24 +608,19 @@ void httpmail_destroy(void)
  */
 int httpmail_authenticate_user(char *user, char *pass)
 {
-  /* Allocate a new empty request object if we don't have one yet */
-  if (!request) request = ghttp_request_new();
+	/* Allocate a new empty request object */
+	request = ghttp_request_new();
 
 	/* Allocate cookies structure and inbox props if not already allocated */
 	if (!cookies) cookies = (KEYPAIR **) calloc (MAX_COOKIES, sizeof(KEYPAIR *));
-	if (!trash) trash = (URL_STRUCT *) calloc (1, sizeof(URL_STRUCT));
-	if (!smtpurl) smtpurl = (URL_STRUCT *) calloc (1, sizeof(URL_STRUCT));
-	/* if (!inbox_props) inbox_props = (FOLDER_STRUCT *) calloc(1,
-	   sizeof(FOLDER_STRUCT)); */
+	if (!inbox_props) inbox_props = (FOLDER_STRUCT *) calloc(1, sizeof(FOLDER_STRUCT));
 
 	if (UserName) free(UserName); /* get rid of any stale username */
 	UserName = (char *) malloc( (strlen(user) + 1) * sizeof(char) );
+	
 	/* Set username/password for authorization-header */	
 	strlcpy(UserName, user, (strlen(user) + 1));
-
-	if (PassWord) free(PassWord);
-	PassWord = (char *) malloc( (strlen(pass) + 1) * sizeof(char) );	
-	strlcpy(PassWord, pass, (strlen(pass) + 1));
+	strlcpy(PassWord, pass, N_PASSWORD);
 
 	return dologin(request);
 }
@@ -811,17 +636,12 @@ int validate_username(char *user)
 
   if (!domain) domain=index(user,'%'); /* check for % sign as well */
   if (!domain) {
-    return E_LOGIN_NO_DOMAIN; /* neither an @ nor % was there */
+	    return E_LOGIN_NO_DOMAIN; /* neither an @ nor % was there */
   } else {
 		int i;
 		*domain='@'; /* make sure the @ sign is there, overwrite any % */
 		domain+= 1;
-		/* first we will ignore the folder section */
-		if (folder_pos) *folder_pos='\0';
-		/* now check if we are allowed to login */
-		if (!allowed_user(user))
-		  return E_LOGIN_NOTALLOWED;
-		/* now check for a server corresponding to the domain */
+		if (folder_pos) *folder_pos='\0'; /* ignore the folder section for the moment */
 		for( i= 0; i< default_servers && !serverurl; i++ ){
 		  /* 20030128 RJVB: do case-insensitive comparison of the servername...! */
 		  if( strcasecmp( default_server[i].domain, domain)== 0 ){
@@ -830,7 +650,7 @@ int validate_username(char *user)
 		}
 		if (folder_pos) *folder_pos='/'; /* put folder back */
   }
-  
+	
   if( !serverurl ){
 	    return E_LOGIN_UNRECOGNISED_DOMAIN;
   }
@@ -838,8 +658,107 @@ int validate_username(char *user)
 }
 
 
+void httpmail_stat(cmdtable *command)
+{
+	long maildrop_size	 = 0,
+		n_not_deleted_emails = 0,
+		i;
 
+	/* GET MESSAGES LIST */
+	if (inbox_props->href) {
+		getlist(request);
+		listdownloaded = 1;
+		for (i = 0; i < inbox_props->listcount; i++) {
+			  /* 20030130 RJVB: FIXME: deleted mails seem to show up as empty messages, at the end of the list
+				 \ can we just not increment n_not_deleted_emails when length==0, or only when there is no more
+				 \ non-zero-length message *after* the current one?
+				 \ But in fact, visiblecount should not have included empty
+				 messages! */
+		      /* 20030421 DAVE: fixed visiblecount inside xmlstuff.c. stat
+				 \ and list run update_visiblecount before filling the message
+				 \structs now. Added a check if running in debug mode also.
+				 */
+			n_not_deleted_emails++;
+			maildrop_size += inbox_props->list_props[i]->length;
+#if DEBUG
+			if (!inbox_props->msg_props[i]->length)
+			  fprintf(stderr, "warning: msg %lu has length 0, that is strange\n", i);
+#endif
 
+		}
+	}
+
+	snprintf(buffer, N_BUFFER, "+OK %lu %lu", n_not_deleted_emails, maildrop_size);
+	register_command( command, buffer, 0, NULL, NULL );
+	PSOUT(buffer);
+	PCRLF;
+}
+
+void httpmail_list(cmdtable *command, char *arg)
+{
+	long i;
+	short cache_flag=1;
+
+	if ( ( (!listdownloaded || (time(NULL)-time_getlist > GETLIST_TIMEOUT) ) && inbox_props->href) ) {
+		getlist(request);
+		listdownloaded = 1;
+		cache_flag--;
+	}
+
+	if (arg == NULL) {
+	    if (!cache_flag) {
+		    PSOUT("+OK Mailbox scan listing follows");
+		} else {
+		    PSOUT("+OK Cached mailbox scan listing follows");
+		}
+		PCRLF;
+ 		for (i = 0; i < inbox_props->listcount; i++) {
+ 			snprintf(buffer, N_BUFFER, "%lu %u\015\012", (i + 1), inbox_props->list_props[i]->length);
+			PSOUT(buffer);
+		}
+		PSOUT(".");
+		PCRLF;
+		return;
+	}
+ 	if ((i = strtoul(arg, NULL, 10)) && (i <= inbox_props->listcount) && (i > 0)) {
+ 		snprintf(buffer, N_BUFFER, "+OK %lu %u", i, inbox_props->list_props[i - 1]->length);
+		register_command( command, buffer, 1, arg, NULL );
+		PSOUT(buffer);
+		PCRLF; 
+	} else {
+		register_command( NULL, NULL, 0, NULL, NULL );
+		PSOUT("-ERR No such message");
+		PCRLF;
+	}
+}
+
+void httpmail_rset(cmdtable *command)
+{
+	register_command( NULL, NULL, 0, NULL, NULL );
+	PSOUT("+OK Mailbox has been reset");
+	PCRLF;
+}
+
+void httpmail_delete(cmdtable *command, char *arg)
+{
+	long i;
+
+	register_command( NULL, NULL, 0, NULL, NULL );	
+ 	if ((i = strtoul(arg, NULL, 10)) && (i <= inbox_props->listcount) && (i > 0)) {
+		if (delmessage(request, i-1)) {
+			PSOUT("+OK Message deleted");
+			PCRLF;
+		}
+		else {
+			PSOUT("-ERR Error deleting message");
+			PCRLF;
+		}
+	}
+	else {
+		PSOUT("-ERR No such message");
+		PCRLF;
+	}
+}
 
 char *gen_uidl(char *dest, char *source)
 {
@@ -856,6 +775,49 @@ char *gen_uidl(char *dest, char *source)
 	return dest;
 }
 
+void httpmail_uidl(cmdtable *command, char *arg)
+{
+	char buf[32];
+	long i;
+
+	if (!listdownloaded && inbox_props->href) {
+		getlist(request);
+		listdownloaded = 1;
+	}
+
+	if (arg == NULL) {
+	  char *response= (char*) calloc(inbox_props->listcount* N_BUFFER+2, sizeof(char));
+	  snprintf(buffer, N_BUFFER, "+OK unique-id listing follows\015\012");
+    if (response) strcat(response, buffer);
+	  PSOUT(buffer);
+ 		for (i = 0; i < inbox_props->listcount; i++) {
+ 			snprintf(buffer, N_BUFFER, "%lu %s\015\012", (i + 1), gen_uidl(buf, inbox_props->list_props[i]->href));
+			if( response ){
+				strcat( response, buffer );
+			}
+			PSOUT(buffer);
+		}
+		PSOUT(".");
+		if( response ){
+			strcat( response, "." );
+		}
+		PCRLF;
+		register_command( command, response, 0, NULL, NULL );
+		if( response ){
+			free(response);
+		}
+		return;
+	}
+ 	else if ((i = strtoul(arg, NULL, 10)) && (i <= inbox_props->listcount) && (i > 0)) {
+ 		snprintf(buffer, N_BUFFER, "+OK %lu %s\015\012", i, gen_uidl(buf, inbox_props->list_props[i - 1]->href));
+		PSOUT(buffer);
+	/* Don't need the PCRLF here as buffer already has CR/LF added */
+	} else {
+		register_command( NULL, NULL, 0, NULL, NULL );
+		PSOUT("-ERR No such message");
+		PCRLF;
+	}
+}
 
 char *next_eol( const char *line, int *len )
 { char *l= NULL;
@@ -975,6 +937,50 @@ char *send_message( char *msg, char *response )
 	return(response);
 }
 
+void httpmail_retrieve(cmdtable *command, char *arg)
+{
+  long i;
+
+  if ((i = strtoul(arg, NULL, 10)) && (i <= inbox_props->listcount) && (i > 0)) {
+	top_lines = -1;
+#ifdef DEBUG
+	fprintf(stderr, "resetting pastheader from value %d\n", pastheader);
+#endif
+	pastheader=0; /* this is just as dodgey as your "top_lines" ;) */
+	getmessage(request, i-1);
+  }
+  else {
+	PSOUT("-ERR No such message");
+	PCRLF;
+	register_command( NULL, NULL, 0, NULL, NULL );
+  }
+}
+
+/* 20020923 RJVB: implementation for the top command, as I understood it from a true pop server */
+void httpmail_top(cmdtable *command, char *arg, char *arg2)
+{
+  unsigned long i;
+  long lines= strtol(arg2, NULL, 10);
+
+  if (lines < 0) {
+	/* For an invalid lines count, just return the whole body */
+	httpmail_retrieve(command,arg);
+  } else {
+	if ((i = strtoul(arg, NULL, 10)) && (i <= inbox_props->listcount) && (i > 0)) {
+	  top_lines = lines;
+	  cur_message = i-1;
+#ifdef DEBUG
+	  fprintf(stderr, "resetting pastheader from value %d\n", pastheader);
+#endif
+	  pastheader=0; 
+	  getmessage(request, cur_message);
+	} else {
+	  PSOUT("-ERR No such message");
+	  PCRLF;
+	  register_command( NULL, NULL, 0, NULL, NULL );
+	}
+  }
+}
 
 /* This function does the generic work of preparing a http request and sending
  * it. It handles any proxy servers that have been specified on the command
@@ -983,8 +989,8 @@ char *send_message( char *msg, char *response )
  * hotmail gives, like when you try to enter an incorrect password for an
  * account many times, perhaps better error reporting will be required here in
  * the future. */
-void prepare_and_send(ghttp_request *request, char *url)
-{
+static void prepare_and_send(ghttp_request *request, char *url)
+{ 
    ghttp_set_header(request, http_hdr_User_Agent, myAgent); 
    ghttp_set_header(request, http_hdr_Connection, "Keep-Alive"); 
    if (proxy) ghttp_set_proxy(request, proxy);
@@ -1009,113 +1015,4 @@ void prepare_and_send(ghttp_request *request, char *url)
    /* here you can see the status returned along with the "reason" */
    printf("Status returned: %d, reason: %s\n",ghttp_status_code(request), ghttp_reason_phrase(request));
 #endif
-   return;
-}
-
-/* This function checks a access list specified as argument to the daemon in
- * order to see if the user trying to login is allowed to use hotwayd on this
- * system. This code is based on a patch provided by Domingo Alczar. */
-static int allowed_user(char *user) {
-  FILE *f;
-  char buf[1000];
-
-  /* if there is no access list specified then anyone can login */
-  if (access_list == NULL)
-    return 1;
-
-  /* otherwise open the file and check the list */
-  f=fopen(access_list, "r");
-  if (f != NULL) {
-    while (!feof(f)) {
-      buf[0] = '\0';
-      fscanf(f, "%999s", &buf);
-      if (buf && user && !strcmp(buf, user)) {
-	fclose(f);
-	return 1;	  
-      }
-    }
-    fclose(f);
-  }
-  return 0;
-}
-
-char *grow_and_copy(char *dest, int curpos, char *src, int i)
-{
-  if (dest==NULL) curpos=0;
-
-  if(!(dest = realloc(dest, curpos+i+1)))
-    return NULL;
-
-  dest[curpos]='\0'; /* dest must be null terminated */
-  strlcat(dest, src, curpos+i+1);
-
-  return dest;
-}
-
-int proxy_sanity_check(void) {
-  if (proxy_username && !proxy_password) {
-	free(proxy_username);
-	printf("Invalid args: Proxy user name supplied, but no password!\n");
-	return 0;
-  } else if (!proxy_username && proxy_password) {
-	free(proxy_password);
-	printf("Invalid args: Proxy password supplied but no user name!\n");
-	return 0;	  
-  } else if ((proxy_username || proxy_password) && !proxy) {
-	if (proxy_username) free(proxy_username);
-	if (proxy_password) free(proxy_password);
-	printf("Invalid args: Proxy user name or password supplied but no proxy given\n");
-	return 0;
-  }	
-  return 1; /* arguments are sane */
-}
-
-void set_proxy(char *proxy_url) {
-  proxy = proxy_url;
-}
-
-void set_proxy_username(char *p_uname) {
-  proxy_username = p_uname;
-}
-
-void set_proxy_password(char *p_pword) {
-  proxy_password = p_pword;
-}
-
-void set_access_list(char *a_list) {
-  access_list = a_list;
-}
-
-/* little function for parsing command line arguments */
-int str2val(register const char *str, register const char *what,
-	    register int mi, register int ma)
-{
-  register const char *cp;
-  register int val;
-  char *ep;
-
-  if (str[0] == '0' && (str[1] == 'x' || str[1] == 'X')) {
-    cp = str + 2;
-    val = (int)strtol(cp, &ep, 16);
-  } else
-    val = (int)strtol(str, &ep, 10);
-  if (*ep != '\0') {
-    fprintf(stderr, "%s: \"%s\" bad value for %s \n",
-	    prog, str, what);
-    exit(1);
-  }
-  if (val < mi && mi >= 0) {
-    if (mi == 0)
-      fprintf(stderr, "%s: %s must be >= %d\n",
-	      prog, what, mi);
-    else
-      fprintf(stderr, "%s: %s must be > %d\n",
-	      prog, what, mi - 1);
-    exit(1);
-  }
-  if (val > ma && ma >= 0) {
-    fprintf(stderr, "%s: %s must be <= %d\n", prog, what, ma);
-    exit(1);
-  }
-  return (val);
 }
