@@ -22,18 +22,13 @@
  * 02111-1307, USA.
  */
 
+#include <pwd.h>
 #include <string.h>
 #include "httpmail.h"
 #include "hotwayd.h"
-#include "cache.h"
 #include "inet.h"
 #include <sys/time.h>
 #include "libghttp-1.0.9-mod/ghttp.h"
-#include "libghttp-1.0.9-mod/http_resp.h"
-#include "libghttp-1.0.9-mod/http_global.h"
-#include "libghttp-1.0.9-mod/http_req.h"
-#include "libghttp-1.0.9-mod/http_lazy.h"
-#include "commands_pop3.h"
 
 #undef AUTHORIZATION
 #define AUTHORIZATION 10
@@ -44,23 +39,43 @@
 #define AUTHORIZATION_TIMEOUT 15
 #define TRANSACTION_TIMEOUT   30
 
-static int state; /* current pop3 state */
-static int timeout, timed_out = 0;
+#define KG_STATE_UNCACHE -1
+#define KG_STATE_CACHE 0
+#define KG_STATE_READ 1
+#define KG_STATE_DUMP 2
+
+#define LAZY_BUFSIZE ((8 * 1024) - 1)
 
 /* Global variables */
+int  state;
 char input[N_BUFLEN];
 char username[N_BUFLEN];
+struct passwd *userpw;
+int timeout, timed_out = 0;
+
+int cur_message = 0;
+int top_lines = -1;
+int pastheader = 0;
 int log_level = 1; /* log level of 0-2 depending how much detail you want */
 char *prog; /* program name */
 
-/* httpmail_ functions */
+/* extern FILE *stdin; */
 
+extern int lazy_set(handler_t);
+extern FOLDER_STRUCT *inbox_props;
+extern int validate_username(char *);
+extern cmdtable *cached_command(cmdtable *command, int argc, char **argv);
+extern char *proxy;
+extern char *proxy_username;
+extern char *proxy_password;
 char *folder=NULL; /* which folder of our mailbox to work on */
 const char *default_folder="inbox";
 char buffer[N_BUFFER];
-extern char version[]; /* version number of hotway, set in VERSION */
 
 void usage(void);
+int args_sanity_check(void);
+static str2val(register const char *str, register const char *what,
+	       register int mi, register int ma);
 
 /****** MISC FUNCTIONS ******/
 /* get the next command from the client 
@@ -104,10 +119,10 @@ void not_spoken_cmd( cmdtable *c, int argc, char *argv[])
 int stat_cmd_no= 0;
 
 void pass_cmd( cmdtable *c, int argc, char *argv[])
-{
-  extern cmdtable commands[];
+{ extern cmdtable commands[];
+ int ret; 
  
-  register_command(NULL, NULL, 0, NULL, NULL);
+ register_command(NULL, NULL, 0, NULL, NULL);
   if (username[0] == '\0') {
     PSOUT("-ERR Please log in with USER first");
     PCRLF;
@@ -122,7 +137,7 @@ void pass_cmd( cmdtable *c, int argc, char *argv[])
 #ifdef DEBUG
   fprintf(stderr,"attempting to login with %s\n",username);
 #endif
-  switch (httpmail_authenticate_user(username, argv[1])) {
+  switch (ret=httpmail_authenticate_user(username, argv[1])) {
   case INVALIDUSER:
     if (log_level > 0)
       LOG("AUTH failed, hotmail said username %s invalid or server too busy, host %s\n", username, inet_client_name);
@@ -135,18 +150,19 @@ void pass_cmd( cmdtable *c, int argc, char *argv[])
     PSOUT("-ERR Remote server said password was invalid");
     PCRLF;
     return;
-  case MUSTPAY:
-    if (log_level > 0)
-      LOG("AUTH failed, hotmail said you have to pay for access for user %s invalid, host=%s\n", username, inet_client_name);
-    PSOUT("-ERR Hotmail said you must pay money to have WebDAV access");
-    PCRLF;
-    return;
   case ERROR:
     if (log_level > 0)
       LOG("AUTH failed, unexpected response from hotmail");
     PSOUT("-ERR Remote server returned unexpected status");
     PCRLF;
     return;
+  case 0:
+    if (log_level > 0)
+      LOG("AUTH failed, couldn't find folder %s", folder);
+    snprintf(buffer, N_BUFFER, "-ERR Unable to find folder %s on remote server", folder);
+    PSOUT(buffer);
+    PCRLF;
+    return;
   case 1:
 	break; /* we logged in ok */
   default:
@@ -154,37 +170,22 @@ void pass_cmd( cmdtable *c, int argc, char *argv[])
       LOG("AUTH failed, unexpected response in pass_cmd");
     PSOUT("-ERR Unexpected response in pass_cmd");
     PCRLF;
-    return;    
-  }
-
-  /* now try to open the folder we want to open */
-  switch(open_folder()) {
-  case 0:
-    if (log_level > 0)
-      LOG("AUTH failed, couldn't find folder %s", folder);
-    snprintf(buffer, N_BUFFER, "-ERR Unable to find folder %s on remote server", folder);
-    PSOUT(buffer);
-    PCRLF;
     return;
-  case 1:
-    break; /* we logged in ok */
-  default:
-    if (log_level > 0)
-      LOG("AUTH failed, unexpected response in pass_cmd");
-    PSOUT("-ERR Unexpected response in pass_cmd");
-    PCRLF;
-    return;        
   }
-
+	
   /* mailbox is open-- we go to transaction state
    * According to RFC 1939, a POP server should be able to handle
    * the RETR command immideately after entering the TRANSACTION
    * state. For this to work, we need to do an implicit STAT on the
    * mail store, otherwise RETR will fail. The STAT will also produce
-   * the needed +OK response for the pass command.
+   * the needed +OK response. To add insult to injury, we need to do
+   * the 'main' STAT action (getting the list of messages from the server)
+   * TWICE to make sure we get all the cookies we need to retrieve
+   * messages. So, that httpmail_init() call is nothing more than a
+   * 'bare bones', no output version of STAT.
    */
-  httpmail_init(); /* run once to get cookies */
-  httpmail_stat( &commands[stat_cmd_no] ); /* and again to get +OK response */
+
+  httpmail_stat( &commands[stat_cmd_no] );
   state = TRANSACTION;
 
   /* log it */
@@ -209,9 +210,6 @@ void user_cmd( cmdtable *c, int argc, char *argv[])
   case E_LOGIN_UNRECOGNISED_DOMAIN:
 	PFSOUT( "-ERR domain not supported (must be a hotmail/msn/lycos/spray domain) in %s\n", argv[1]);
 	break;
-  case E_LOGIN_NOTALLOWED:
-    PFSOUT( "-ERR this %s server does not permit %s. Speak to your sysadmin\n", prog, argv[1]);
-    break;
   case E_LOGIN_OK:
 	PSOUT("+OK Username validated, Password required");
 	PCRLF;
@@ -239,14 +237,12 @@ void user_cmd( cmdtable *c, int argc, char *argv[])
 		}
 		*folder_pos='\0'; /* now kill the folder part of the username */
 	  }
-#if 0
 	  /* this needs to change, if it is a @hotmail.com address then remove the
 		 domain from username, if it is @msn.com then keep the domain */
-	  if (domain_begin && (strcasecmp("@hotmail.com",domain_begin) == 0)) {
+	  if ((strcasecmp("@hotmail.com",domain_begin) == 0)) {
 		/* remove the domain from username for @hotmail.com addresses */
 		*domain_begin = '\0'; 
 	  }
-#endif
 	  register_command(NULL, NULL, 0, NULL, NULL);
 	}
 	break;
@@ -307,7 +303,7 @@ void retr_cmd( cmdtable *c, int argc, char *argv[])
     PCRLF;
     return;
   }
-  httpmail_retr(c, argv[1]);
+  httpmail_retrieve(c, argv[1]);
 }
 
 void dele_cmd( cmdtable *c, int argc, char *argv[])
@@ -317,7 +313,7 @@ void dele_cmd( cmdtable *c, int argc, char *argv[])
     PCRLF;
     return;
   }
-  httpmail_dele(c, argv[1]);
+  httpmail_delete(c, argv[1]);
 }
 
 void noop_cmd( cmdtable *c, int argc, char *argv[])
@@ -371,11 +367,7 @@ cmdtable commands[] = 	{
 };
 
 
-/* Function: main_loop()
- * Wait for POP3 commands. Takes care of transition between states (defined in
- * POP3 standard) and uses the command jump table (commands[]) to determine
- * which function is called upon a given command.
- */
+/******* MAIN LOOP ******/
 void main_loop(void)
 {
   int   argc;
@@ -383,6 +375,7 @@ void main_loop(void)
     *x;
   int   cmd_implemented;
   cmdtable *c;
+  extern char version[]; /* version number of hotway, set in VERSION */
 
   state = AUTHORIZATION; 
   username[0] = '\0';
@@ -391,7 +384,7 @@ void main_loop(void)
   if (inet_server_domain[0] != '\0') {
     PFSOUT("%s ", inet_server_domain);
   }
-  PFSOUT("%s v%s -> The POP3-HTTPMail Gateway.", prog, version);
+  PFSOUT("hotwayd v%s -> The POP3-HTTPMail Gateway.", version);
   PFSOUT(" Server on %s active.\015\012", inet_server_name);
   PFLUSH;
 		
@@ -511,38 +504,217 @@ void main_loop(void)
   }
 }
 
+char *grow_and_copy(char *dest, int curpos, char *src, int i)
+{
+  if(!(dest = realloc(dest, curpos+i+1)))
+    return NULL;
 
-/* Function: main
- * Entry point for hotwayd. Takes some command line arguments, processes them
- * and then calls main_loop() to wait for commands. Once main_loop() returns
- * we cleanup our mess and exit.
+  memcpy(dest+curpos, src, i);
+  dest[curpos+i] = '\0';
+
+  return dest;
+}
+
+/*
+ * Function lazy_dump
+ *
+ * This function is used by lazy_handler to print out the buffer collected by
+ * lazy_handler line by line. The reason we do it like this is two fold. First
+ * so that we can implement TOP and second so that if we are downloading a big
+ * message it means the mail reader won't time out waiting for hotwayd to spew
+ * the message out in a big chunk. It also makes the problem more complex so
+ * why not :-)
  */
+
+int lazy_dump(char *pos, int len, int *curpos_p, int *curline_p, int maxlines) 
+ {
+  char foo[LAZY_BUFSIZE+1];
+
+  char *cureol = NULL;
+
+  int curpos = *curpos_p;
+  int curline = *curline_p;
+
+  int i;
+
+  int just_sent_line = 1;
+  int will_send_line = 0;
+
+  foo[LAZY_BUFSIZE] = '\0';
+
+  if(len == -1)
+    len = strlen(pos);
+
+  /* hack hack hack */
+  if(maxlines == 0 && curline == 0)
+    curline = -1;
+
+  /* flush from pos */
+  while(((maxlines == -1) || (curline < maxlines)) && (curpos < len)) {
+	char *next_n = NULL;
+	char *next_r = NULL;
+
+	next_n=strstr(pos+curpos, "\n");
+	next_r=strstr(pos+curpos, "\r");
+
+	if (next_r && (*(next_r+1) == '\n')) next_r++; /* handle \r\n */
+
+	/* check which is the next to come, a \r, a \n or a \r\n */
+	if (next_r && next_n && (next_r < next_n)) {
+	  cureol = next_r;
+	} else if (next_n) {
+	  cureol = next_n;
+	} else {
+	  cureol = NULL;
+	}
+	  
+	if(cureol) 
+      i = MIN(cureol-(pos+curpos)+1, LAZY_BUFSIZE);
+
+    if(cureol && (i == (cureol-(pos+curpos)+1))) {
+
+      if(maxlines && pastheader) {
+		curline++; /* we are past the header, start counting lines */
+	  }
+
+      if( !pastheader && /* still in header */
+	  just_sent_line &&  /* just flushed to a \r\n */
+	  ( (pos[curpos] == '\r' && pos[curpos+1] == '\n') ||
+	    (pos[curpos] == '\n') ) ) { /* and find ourselves at another \r\n */ 
+		pastheader++; /* right, now we can start counting lines.. */
+	if(maxlines == 0 && curline == -1)
+	   curline = 0;
+      }
+      
+      will_send_line = 1;
+    } else {
+      i = MIN(len-curpos, LAZY_BUFSIZE);
+      just_sent_line = 0;
+    }
+
+    memcpy(foo, pos+curpos, i);
+    foo[i] = '\0';
+
+    if(just_sent_line && foo[0] == '.')
+	printf(".");
+
+    printf("%s", foo);
+
+    just_sent_line = will_send_line;
+
+    will_send_line = 0;
+
+    curpos += i;
+  }
+
+  *curline_p = curline;
+  *curpos_p = curpos;
+
+  if(curpos >= len) {
+	return 1;
+  }
+  if(curline >= maxlines) {
+    return 2;
+  }
+  return 0;
+}
+
+/*
+ * Function lazy_handler
+ *
+ * We use this function to progressively read a message from the mail
+ * server. It works by filling an internal buffer (of LAZY_BUFSIZE) and
+ * sending chunks to lazy_dump to be output. 
+ * 
+ * what i said about "it was a mess when i got here" being no excuse..
+ * i was lying through my libghttping[1] teeth
+ *  -jbm
+ * [1]: yes, it's now officially profanity.
+ */
+   
+
+int lazy_handler(char *pos, int len, int totallen, int fd)
+{
+  char foo[LAZY_BUFSIZE + 1];
+  char *dest = NULL;
+  int curpos = 0;
+  int i;
+  int kg = KG_STATE_READ;
+  int curline = 0;
+  int curlen = 0;
+  foo[LAZY_BUFSIZE] = '\0';
+
+  PSOUT("+OK Going to Lazy-download that one\r\n");
+
+  if(!(dest = grow_and_copy(dest, 0, pos, len))) {
+    printf("-ERR OR damn, i dunno: %s\r\n", strerror(errno));
+    return -1;
+  }
+  curlen += len;
+
+  while(kg > KG_STATE_CACHE) {
+    if(kg == KG_STATE_READ) {
+      /* better not to call read at end of buffer so we'll use totallen which
+       * was discovered from Content-Length of headers to see if we have
+       * finished reading all of the message. It is also possible to use
+       * select here, but I'd rather leave it as the default timeout incase
+       * someone has a very slow connection */
+      if (curlen < totallen)
+	i = read(fd, foo, LAZY_BUFSIZE);
+      else
+	i = 0;
+      if(i < 0) {
+	kg = KG_STATE_UNCACHE;
+	continue;
+      }
+
+      if(i == 0)
+	kg = KG_STATE_DUMP;
+
+      foo[i] = '\0';
+
+      if(!(dest = grow_and_copy(dest, curlen, foo, i))) { 
+	printf("-ERR OR damn, i dunno: %s\r\n", strerror(errno));
+	return 0;
+      }
+      curlen += i;
+    }
+
+    if(kg == KG_STATE_DUMP || kg == KG_STATE_READ) {
+      i = lazy_dump(dest, curlen, &curpos, &curline, top_lines);
+
+      if(kg == KG_STATE_DUMP && i == 1)
+	kg = KG_STATE_CACHE;
+      if(i == 2) /* to handle TOP as well as RETR -- top probably won't hit DUMP*/
+	kg = KG_STATE_UNCACHE;
+
+    }
+  }
+
+  PSOUT(".\r\n"); /* required by POP3 standard at end of message */
+
+  free(inbox_props->msg_props[cur_message]->message);
+
+  if(kg == KG_STATE_CACHE)
+    inbox_props->msg_props[cur_message]->message = dest;
+  else
+    free(dest);
+
+  return 1;
+}
+
+/****** MAIN *****/
 int main(int argc, char **argv)
 {
   cmdtable *c= commands;
   int		opt;
-  char *prog_name;
 
-  nice(20); /* give hotwayd the lowest priority in case it hangs and hogs CPU */
-
-  prog_name = strrchr(argv[0], '/'); /* remove slashes from progname */
-
-  if (prog_name && prog_name+1 != '\0')
-    prog=strdup(prog_name+1);
-  else
-    prog=strdup(argv[0]);
+  prog=strdup(argv[0]);
   
-  while ((opt = getopt(argc, argv, "a:hp:ru:q:l:v")) != -1) {
-        char *access_list, *proxy, *proxy_username, *proxy_password;
+  while ((opt = getopt(argc, argv, "hp:u:q:l:")) != -1) {
 
 	switch (opt) {
 
-	case 'a':
-	  access_list = (char *)malloc(strlen(optarg)+1);
-	  strlcpy(access_list, optarg, strlen(optarg)+1);
-	  set_access_list(access_list);
-	  break;
-	  
 	case 'h': /* display the usage page */
 	  usage();
 	  return 0;
@@ -554,29 +726,18 @@ int main(int argc, char **argv)
 	case 'p': /* setup proxy usage */
 	  proxy = (char *) malloc (strlen(optarg)+1);
 	  strlcpy(proxy, optarg, strlen(optarg)+1);
-	  set_proxy(proxy);
 	  break;
 
-	case 'r':
-	  set_readupdate_flag(1);
-	  break;
-	  
 	case 'u':
 	  proxy_username = (char *) malloc (strlen(optarg)+1);
 	  strlcpy(proxy_username, optarg, strlen(optarg)+1);
-	  set_proxy_username(proxy_username);
 	  break;
 
 	case 'q':
 	  proxy_password = (char *) malloc (strlen(optarg)+1);
 	  strlcpy(proxy_password, optarg, strlen(optarg)+1);
-	  set_proxy_password(proxy_password);
 	  break;
 
-	case 'v':
-	  PFSOUT("%s v%s - http://hotwayd.sourceforge.net/\n", prog, version);
-	  return 0;	  
-	  
 	default:
 	  usage();
 	  return -1;
@@ -584,7 +745,7 @@ int main(int argc, char **argv)
   }
 
   /* check validity of command line arguments */
-  if (!proxy_sanity_check()) {
+  if (!args_sanity_check()) {
 	usage();
 	return -1;
   }
@@ -603,10 +764,8 @@ int main(int argc, char **argv)
   }
 
   main_loop();
-
-  /* ok let's clean up our mess as much as we can :-) */
+	
   httpmail_destroy();
-  inboxprops_destroy();
   CLOSELOG;
   DPRINTF("exiting.\n");
   return(0);
@@ -614,15 +773,65 @@ int main(int argc, char **argv)
 
 
 void usage(void) {
-  printf("Usage: %s [options and arguments]\n"
-	 "  -a <accesslist> specify a file listing permitted HTTPMail accounts\n"
-	 "  -h print usage page\n"
-	 "  -l <loglevel> specify the logging level (0-2)\n"
-	 "  -p [proxy:port] specify proxy\n"
-	 "  -r set messages as read after downloading\n"
-	 "  -u <username> specify proxy username\n"
-	 "  -q <password> specify proxy password\n"
-	 "  -v display version information\n"
-	 "Example: %s -p http://proxy:8080 -u dave -q proxypass\n"
-	 "Note: proxy can be specified without a username or password\n", prog, prog);
+  printf("Usage: hotwayd [options and arguments]\n"
+	     "  -h print usage page\n"
+	     "  -l <loglevel> specify the logging level (0-2)\n"
+	     "  -p [proxy:port] specify proxy\n"
+	     "  -u [username] specify proxy username\n"
+	     "  -q [password] specify proxy password\n"
+	     "Example: hotwayd -p http://proxy:8080 -u dave -q proxypass\n"
+	     "Note: proxy can be specified without a username or password\n");
+}
+
+
+int args_sanity_check(void) {
+  if (proxy_username && !proxy_password) {
+	free(proxy_username);
+	printf("Invalid args: Proxy user name supplied, but no password!\n");
+	return 0;
+  } else if (!proxy_username && proxy_password) {
+	free(proxy_password);
+	printf("Invalid args: Proxy password supplied but no user name!\n");
+	return 0;	  
+  } else if ((proxy_username || proxy_password) && !proxy) {
+	if (proxy_username) free(proxy_username);
+	if (proxy_password) free(proxy_password);
+	printf("Invalid args: Proxy user name or password supplied but no proxy given\n");
+	return 0;
+  }	
+  return 1; /* arguments are sane */
+}
+
+
+static str2val(register const char *str, register const char *what,
+    register int mi, register int ma)
+{
+        register const char *cp;
+        register int val;
+        char *ep;
+
+        if (str[0] == '0' && (str[1] == 'x' || str[1] == 'X')) {
+                cp = str + 2;
+                val = (int)strtol(cp, &ep, 16);
+        } else
+                val = (int)strtol(str, &ep, 10);
+        if (*ep != '\0') {
+                fprintf(stderr, "%s: \"%s\" bad value for %s \n",
+			prog, str, what);
+                exit(1);
+        }
+        if (val < mi && mi >= 0) {
+                if (mi == 0)
+                        fprintf(stderr, "%s: %s must be >= %d\n",
+                            prog, what, mi);
+                else
+                        fprintf(stderr, "%s: %s must be > %d\n",
+                            prog, what, mi - 1);
+                exit(1);
+        }
+        if (val > ma && ma >= 0) {
+                fprintf(stderr, "%s: %s must be <= %d\n", prog, what, ma);
+                exit(1);
+        }
+        return (val);
 }
