@@ -28,29 +28,27 @@
 #include <unistd.h>
 #include <errno.h>
 
-#define N_BUFLEN 1024 /* used for reading in commands to pop3/imap/smtp proxy */
+#define N_BUFLEN 200
 #define N_BUFFER 1024
 #define N_MYBUF 1024
 #define N_PAIR 1024
+#define N_XML_LENGTH 16
+#define N_PASSWORD 32
 
 #define PAM_SERVICE_NAME "hotway"
 
 #define SYSLOGNAME "hotway"
 #define SYSLOGFACILITY LOG_LOCAL7
 
-#define COPY 1
-#define MOVE 2
-#define TRASH 3
-
 #ifndef MIN
 #	define MIN(a,b)                (((a)<(b))?(a):(b))
 #endif
 
 /* the following are login errors */
-#define E_LOGIN_OK 0
+#define E_LOGIN_NO_DOMAIN 0
 #define E_LOGIN_UNRECOGNISED_DOMAIN 1
-#define E_LOGIN_NO_DOMAIN 2
-#define E_LOGIN_NOTALLOWED 3
+#define E_LOGIN_OK 2
+
 
 /* #define PSOUT(args...) printf( args)	*/
 #define PSOUT(args) fputs( args, stdout)
@@ -76,7 +74,7 @@ size_t strlcat(char *, const char*, size_t);
 #define MAXARGC 4
 
 typedef struct cmdtable {
-		char name[20];
+		char name[10];
 		int  states;
 		void  (*handler)(struct cmdtable *, int, char *[]);
 		int argc;
@@ -85,3 +83,18 @@ typedef struct cmdtable {
   char *response;
 } cmdtable;
 
+/* httpmail_ functions */
+long httpmail_nemails(void);
+void httpmail_destroy(void);
+int httpmail_authenticate_user(char *, char *);
+void httpmail_init(void);
+void httpmail_stat(cmdtable *);
+void httpmail_list(cmdtable *,char *);
+void httpmail_rset(cmdtable *);
+void httpmail_retrieve(cmdtable *,char *);
+void httpmail_delete(cmdtable *,char *);
+char *gen_uidl(char *, char *);
+void httpmail_uidl(cmdtable *,char *);
+void httpmail_top(cmdtable *,char *, char *);
+
+extern void register_command( cmdtable *command, char *response, int argc, char *arg1, char *arg2 );
