@@ -216,9 +216,6 @@ ghttp_set_type(ghttp_request *a_request, ghttp_type a_type)
     {
     case ghttp_type_lazy:
       a_request->req->type = http_req_type_lazy;
-      break;
-    case ghttp_type_lazy_buf:
-      a_request->req->type = http_req_type_lazy_buf;
       break;      
     case ghttp_type_get:
       a_request->req->type = http_req_type_get;
