@@ -30,8 +30,6 @@
 #include "http_resp.h"
 #include "http_global.h"
 #include "http_req.h"
-#include "ghttp.h"
-#include "http_lazy.h"
 
 typedef enum header_state_tag
 {
@@ -375,7 +373,7 @@ http_resp_read_headers(http_resp *a_resp, http_trans_conn *a_conn, http_req_type
 	  goto ec;
 	}
     }
-      if (type == http_req_type_lazy || type == http_req_type_lazy_buf) {
+      if (type == http_req_type_lazy) {
 	char *pos;
 	int len = 0;
 
@@ -386,7 +384,7 @@ http_resp_read_headers(http_resp *a_resp, http_trans_conn *a_conn, http_req_type
 	  len = a_conn->io_buf + a_conn->io_buf_alloc - pos;
 	}
 	
-	return lazy_read(a_resp, a_conn, pos, len, type);
+	return lazy_read(a_resp, a_conn, pos, len);
       }
 
 
