@@ -31,8 +31,6 @@ vim:ts=3:sw=3:
 #include "libghttp-1.0.9-mod/ghttp.h"
 #include "httpmail.h"
 #include "hotwayd.h"
-#include "commands.h"
-#include "xmlstuff.h"
 
 const char *dir_names_request = "\
 <?xml version=\"1.0\"?>\r\n\
@@ -81,27 +79,12 @@ const char *msg_props_request = "\
     </D:prop>\r\n\
 </D:propfind>";
 
-const char *propupdate_read_start = "\
-<?xml version=\"1.0\"?>\r\n\
-<D:propertyupdate xmlns:D=\"DAV:\" xmlns:hm=\"urn:schemas:httpmail:\" xmlns:m=\"urn:schemas:mailheader:\">\r\n\
-  <D:set>\r\n\
-    <D:prop>\r\n\
-      <hm:read>";
-
-const char *propupdate_read_end = "</hm:read>\r\n\
-    </D:prop>\r\n\
-  </D:set>\r\n\
-</D:propertyupdate>";
-
-
 xmlDocPtr doc = NULL;
 xmlNsPtr  ns  = NULL;
 
 static xmlNodePtr next_node(xmlNodePtr node);
 static xmlNodePtr child_node(xmlNodePtr node);
 static void cp_href(xmlNodePtr hrefptr,  FOLDER_STRUCT *folder_props);
-static FOLDER_STRUCT *handle_response_node(xmlNodePtr node, char *foldername, FOLDER_STRUCT *folder_props);
-static int count_folders(xmlNodePtr node);
 
 extern int log_level;
 
@@ -195,7 +178,7 @@ char *findvalue(char *xmldoc, char *namesp, char *key) {
 
     /* goto root node  */
     while ( cur && xmlIsBlankNode(cur))
-      if (!(cur=next_node(cur))) return NULL;
+      if (!(cur=next_node(cur))) return NULL;;
 	
 	/* traverse xml tree! */
 	if (!xmlStrcmp(cur->name, (const xmlChar *) "multistatus")) {
@@ -224,186 +207,10 @@ char *findvalue(char *xmldoc, char *namesp, char *key) {
 }
 
 
-FOLDER_STRUCT **folderlist(char *xmldoc, FOLDER_STRUCT **folder_list, int *folder_cnt) {
-  xmlNodePtr cur=NULL;
-  int i;
-
-  *folder_cnt = 0;
-  
-  if (!(cur = initxml(xmldoc, "hm", cur)))
-    return NULL;
-
-  /* goto root node  */
-  while ( cur && xmlIsBlankNode(cur))
-    if (!(cur=next_node(cur)))
-      return NULL;
-
-  if (xmlStrcmp(cur->name, (const xmlChar *) "multistatus"))
-    return NULL;
-
-  /* get number of folders and allocate space in the list for each folder */
-  *folder_cnt = count_folders(cur);
-  if (!(cur=child_node(cur))) return folder_list; /* go down to href/propstat */
-  if (!folder_list) folder_list = (FOLDER_STRUCT **) calloc (*folder_cnt, sizeof(FOLDER_STRUCT *));
-
-  for (i=0;i<*folder_cnt;i++) {
-    /* go to the first response node */
-    while (xmlStrcmp(cur->name, (const xmlChar *) "response"))
-      cur=next_node(cur);
-    if (cur == NULL)
-      return NULL;    
-    folder_list[i] = handle_response_node(cur, NULL, NULL);
-    if (!(cur=next_node(cur)))
-      break;    
-  }
-
-#if 0
-  if (i != (*folder_cnt)-1) {
-    /* urgh.. let's cheat and call folder_cnt i+1 */
-    *folder_cnt = i+1;
-  }
-#endif
-  
-  return folder_list;
-}
-
-/* Function handle_response_node()
- * This function is used to handle the response node of a parsed folder list
- * from the hotmail server. It allocates a folder_props structure if none is
- * passed to the function and fills it in unconditionally if foldername is
- * NULL otherwise it only fills in the details if the folder is called
- * foldername.
- */
-static FOLDER_STRUCT *handle_response_node(xmlNodePtr node, char *foldername, FOLDER_STRUCT *folder_props) {
-  int found_fld=0,found_unread=0,found_visible=0, found=0;
-  xmlNodePtr href=NULL; /* temp store for the href */
-
-  if (xmlStrcmp(node->name, (const xmlChar *) "response"))
-    return NULL;
-  if (!(node=child_node(node))) return NULL; /* go down to href/propstat */
-  if (!folder_props)
-    folder_props = calloc(1, sizeof(FOLDER_STRUCT));
-  
-  do {
-    if (!xmlStrcmp(node->name, (const xmlChar *) "href")) {
-      href=node;
-
-    } else if (!xmlStrcmp(node->name,(const xmlChar *) "propstat")) {
-      int ignore_special = 0;
-
-      if (!(node=child_node(node))) break;
-
-      while (xmlStrcmp(node->name, (const xmlChar *) "prop")) {
-	if (!(node=next_node(node))) break;
-      }
-      
-      if (!(node=child_node(node))) break; /* goto D:isfolder */
-
-      do {
-
-	/* it is possible for the node to be <hm:special> if it is a fixed
-	 * hotmail folder or <D:displayname> if it is a folder added by a
-	 * user to hotmail so we will now check for both */
-	if (!xmlStrcmp(node->name, (const xmlChar *) "special")
-	    && (!node->xmlChildrenNode 
-		|| strlen(xmlNodeListGetString(doc, node->xmlChildrenNode, 1))) < 1)
-	  ignore_special++;
-
-	if ((!folder_props->name || !ignore_special)
-	    && node->xmlChildrenNode /* catch null pointers */
-	    && (!xmlStrcmp(node->name, (const xmlChar *) "special")
-		|| !xmlStrcmp(node->name, (const xmlChar *) "displayname"))
-	    && (!foldername || (!xmlStrcmp(foldername, xmlNodeListGetString(doc, node->xmlChildrenNode, 1))))) {
-
-	  if (folder_props->name)
-	    free(folder_props->name); /* special overrides displayname */
-	  folder_props->name = strdup(xmlNodeListGetString(doc, node->xmlChildrenNode, 1));
-	  found_fld++;
-	} else if (!xmlStrcmp(node->name, (const xmlChar *) "unreadcount")) {
-	  folder_props->unreadcount = atoi(xmlNodeListGetString(doc, node->xmlChildrenNode, 1));
-	  found_unread++;
-	} else if (!xmlStrcmp(node->name, (const xmlChar *) "visiblecount") ) {
-	  folder_props->visiblecount = atoi(xmlNodeListGetString(doc, node->xmlChildrenNode, 1));
-	  found_visible++;
-	}
-	if (node->next != NULL) node=node->next;
-      } while (node->next != NULL);
-      
-      node=node->parent; /* go back to D:prop/D:status node */
-      node=node->parent; /* go back to D:propstat/D:href */
-    }
-
-    if (found_fld && found_unread && found_visible)
-      found=1;
-    else 
-      found_fld=found_unread=found_visible=0;
-    
-    if (node->next != NULL) node=node->next;
-
-  } while (node->next != NULL);
-
-  /* if we didn't find the correct folder then free any allocated space */
-  if (!found) {
-    if (folder_props->name) free(folder_props->name);
-    if (folder_props->href) free(folder_props->href);
-    folder_props=NULL;
-  } else {
-    cp_href(href, folder_props);
-  }
-
-  return folder_props;
-}
-
-/* We need to know how many folders we have before we can allocate space for
- * the folder properties. Hence the purpose of this function is to count the
- * number of folders so we can allocate space for their properties.
- */
-static int count_folders(xmlNodePtr node) {
-  int folder_cnt=0;
-
-  if (node == NULL) return 0;
-
-  /* traverse xml tree! */
-  if (!xmlStrcmp(node->name, (const xmlChar *) "multistatus")) {
-    /* go to the first response node */
-    node=node->children;
-    while (node) {
-      if (!xmlStrcmp(node->name, (const xmlChar *) "response")) {
-	if (!(node=child_node(node))) return folder_cnt; /* go down to href/propstat */
-	do {
-	  if (!xmlStrcmp(node->name,(const xmlChar *) "propstat")) {
-	    if (!(node=child_node(node))) return folder_cnt;
-	    while (xmlStrcmp(node->name, (const xmlChar *) "prop")) {
-	      if (!(node=next_node(node))) return folder_cnt;
-	    }
-	    if (!(node=child_node(node))) return folder_cnt; /* goto D:isfolder */
-	    /* check if it is a folder or not */
-	    do {
-	      if (!xmlStrcmp(node->name, (const xmlChar *) "isfolder")
-		  && atoi(xmlNodeListGetString(doc, node->xmlChildrenNode, 1))) {
-		folder_cnt++;
-		break; /* because we know it is a folder now */
-	      }
-	      if (node->next != NULL) node=node->next;
-	    } while (node->next != NULL);
-	    node=node->parent; /* go back to D:prop/D:status node */
-	    node=node->parent; /* go back to D:propstat/D:href */
-	  }
-	  if (node->next != NULL) node=node->next;
-	} while (node->next != NULL);
-	node=node->parent; /* go back to D:response node */
-      }
-      node=node->next; /* try to go to the next D:response node */
-    } /* end of while loop */
-  }
-  return folder_cnt;
-}
-
 /**
  * Setup the folder_props data structure with folder_props->name - the name of
  * the folder, folder_props->unreadcount - how many messages are unread in the
- * folder, and folder_props->visiblecount - how many messages are in the
- * folder. Returns 1 on success, 0 on failure.
+ * folder, and folder_props->visiblecount - how many messages are in the folder
  **/
 int folderprops(char *xmldoc, char *foldername, FOLDER_STRUCT *folder_props) {
   int found=0;
@@ -422,8 +229,49 @@ int folderprops(char *xmldoc, char *foldername, FOLDER_STRUCT *folder_props) {
 	cur=cur->children;
 	while (cur != NULL && !found) {
 	  if (!xmlStrcmp(cur->name, (const xmlChar *) "response")) {
-	    if (handle_response_node(cur, foldername, folder_props))
-	      found = 1;
+		int found_fld=0,found_unread=0,found_visible=0;
+		xmlNodePtr href=NULL; /* temp store for the href */
+		if (!(cur=child_node(cur))) return found; /* go down to href/propstat */
+		do {
+		  if (!xmlStrcmp(cur->name, (const xmlChar *) "href")) {
+			href=cur;
+		  } else if (!xmlStrcmp(cur->name,(const xmlChar *) "propstat")) {
+			if (!(cur=child_node(cur))) return found;
+			while (xmlStrcmp(cur->name, (const xmlChar *) "prop")) {
+			  if (!(cur=next_node(cur))) return found;
+			}
+			if (!(cur=child_node(cur))) return found; /* goto D:isfolder */
+			do {
+			  /* it is possible for the node to be <hm:special> if it is a fixed
+			   * hotmail folder or <D:displayname> if it is a folder added by a
+			   * user to hotmail so we will now check for both */
+			  if ((!xmlStrcmp(cur->name, (const xmlChar *) "special") || !xmlStrcmp(cur->name, (const xmlChar *) "displayname"))
+				  && (!xmlStrcmp(foldername, xmlNodeListGetString(doc, cur->xmlChildrenNode, 1)))) {
+				folder_props->name = strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
+				found_fld++;
+				cp_href(href, folder_props);
+			  } else if (!xmlStrcmp(cur->name, (const xmlChar *) "unreadcount")) {
+				folder_props->unreadcount = atoi(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
+				found_unread++;
+			  } else if (!xmlStrcmp(cur->name, (const xmlChar *) "visiblecount") ) {
+				folder_props->visiblecount = atoi(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
+				found_visible++;
+			  }
+			  if (cur->next != NULL) cur=cur->next;
+			} while (cur->next != NULL);
+			cur=cur->parent; /* go back to D:prop/D:status node */
+			cur=cur->parent; /* go back to D:propstat/D:href */
+		  }
+
+		  if (found_fld && found_unread && found_visible)
+			found=1;
+		  else 
+			found_fld=found_unread=found_visible=0;
+
+		  if (cur->next != NULL) cur=cur->next;
+
+		} while (cur->next != NULL);
+		cur=cur->parent; /* go back to D:response node */
 	  }
 	  cur=cur->next; /* try to go to the next D:response node */
 	} /* end of while loop */
@@ -557,17 +405,11 @@ void msglengths(char *xmldoc, FOLDER_STRUCT *folder_props )
 		  if (!cur) { xmlFreeDoc(doc); return; }
 		  /* FIXME: should check if both read and getcontentlength have been
 		   * found and if not report error */
-		  if (msg[i]->date) {
-		    free(msg[i]->date);
-		    msg[i]->date = NULL;
-		  }
 		  for(;;) {
 			if (!xmlStrcmp(cur->name, (const xmlChar *) "read"))
 			  (msg[i])->readmark = atoi((const char *)xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
 			else if (!xmlStrcmp(cur->name, (const xmlChar *) "getcontentlength"))
 			  (msg[i])->length = atoi((const char *)xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
-			else if (!xmlStrcmp(cur->name, (const xmlChar *) "date"))
-			  (msg[i])->date = strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
 			if (cur->next == NULL)
 			  break;
 			else
@@ -642,46 +484,12 @@ void update_visiblecount(char *xmldoc, FOLDER_STRUCT *folder_props ) {
 }
 
 
-static void cleanup_folder_props(FOLDER_STRUCT *folder_props)
-{
-  int i;
-  if (folder_props->msg_props) {
-    for (i=0; i<folder_props->visiblecount; i++) {
-      if (folder_props->msg_props[i]->href)
-	free(folder_props->msg_props[i]->href);
-      if (folder_props->msg_props[i]->message)
-	free(folder_props->msg_props[i]->message);
-      if (folder_props->msg_props[i]->header)
-	free(folder_props->msg_props[i]->header);
-      if (folder_props->msg_props[i]->date)
-	free(folder_props->msg_props[i]->date);
-      free(folder_props->msg_props[i]);
-      folder_props->msg_props[i]=NULL;
-    }
-    free(folder_props->msg_props);
-    folder_props->msg_props=NULL;
-  }
-  if (folder_props->unread_props) {
-    free(folder_props->unread_props);
-    folder_props->unread_props=NULL;
-  }
-}
-
 int ParseXMLMsgList(ghttp_request *request, FOLDER_STRUCT *folder_props)
 {
   int i, n;
-  char *src;
-
-  if (folder_props->msg_props || folder_props->unread_props) {
-    /* urgh, let's clean it this up */
-    cleanup_folder_props(folder_props);
-  }
-
-  src = ghttp_get_body(request);
-  remove_to_from_subject (src);
-
-  update_visiblecount(src, folder_props);
   
+  update_visiblecount(ghttp_get_body(request), folder_props);
+
   if ((n = folder_props->visiblecount) > 0) {
 	/* Creates array of MSG_STRUCTs */
 	folder_props->msg_props = (MSG_STRUCT **) calloc(n, sizeof(MSG_STRUCT *));
@@ -697,7 +505,7 @@ int ParseXMLMsgList(ghttp_request *request, FOLDER_STRUCT *folder_props)
 	
 	/* Fill our MSG_STRUCTS */
 	/* RJVB 20020927: necessary to update visiblecount and msg_props if the nr. of messages changed! */
-	msglengths(src, folder_props);
+	msglengths(ghttp_get_body(request), folder_props);
 	
 	/* Build the list of unread messages */
 	folder_props->unreadcount = buildunreadlist(folder_props->msg_props, n, folder_props->unread_props);
@@ -705,55 +513,6 @@ int ParseXMLMsgList(ghttp_request *request, FOLDER_STRUCT *folder_props)
   return 1;
 }
 
-/*#define DEBUG_HEAD 1*/
-
-/* remove to, from and subject contents because they may contain UTF-16 characters and those cause parsing error */
-remove_to_from_subject (char *in)
-{
-  char *p, *q, *r;
-  int i;
-  static const struct {
-    char *start;
-    char *end;
-  } eliminate[] = {
-    {"<m:subject>", "</m:subject>"},
-    {"<m:from>", "</m:from>"},
-    {"<m:to>", "</m:to>"},
-    {NULL, NULL}
-  };
-
-#ifdef DEBUG_HEAD
-  FILE *fp;
-
-  fp = fopen ("/tmp/111.txt", "a");
-  fprintf (fp, "1: %s\n", in);
-#endif
-
-  for (i=0; eliminate[i].start != NULL; i++)
-    {
-      for (r =in;; r = p+strlen(eliminate[i].end))
-        {
-          p = strstr (r, eliminate[i].start);
-          q = strstr (r, eliminate[i].end);
-          if (p != NULL && q != NULL)
-            {
-              p += strlen (eliminate[i].start);
-              while (p < q)
-                *p++ = ' ';
-            }
-          else
-            {
-              break;
-            }
-        }
-    }
-
-#ifdef DEBUG_HEAD
-  fprintf (fp, "2: %s\n", in);
-  fclose (fp);
-#endif
-}
-
 
 /* Use this function to attempt to find the folder foldername and fill in the
  * folder_props->href, folder_props->visiblecount and
@@ -765,25 +524,9 @@ int ParseXMLFolderProps(ghttp_request *request, char *foldername, FOLDER_STRUCT
 
 /* Use this function to find the root address of all folders in a particular
  * msn/hotmail account. It returns the address of the root folder. It also
- * fills in the trash and sendmsg folders href while it does its work - why not :-)
+ * fills in the trash folder href while it does its work - why not :-)
  */
-char *ParseXMLFolderRoot(ghttp_request *request, URL_STRUCT *trash, URL_STRUCT *sendmsg) {
-  trash->href = findvalue(ghttp_get_body(request), "hm", "deleteditems");
-  sendmsg->href = findvalue(ghttp_get_body(request), "hm", "sendmsg");
+char *ParseXMLFolderRoot(ghttp_request *request, FOLDER_STRUCT *folder_props) {
+  folder_props->trash = findvalue(ghttp_get_body(request), "hm", "deleteditems");
   return findvalue(ghttp_get_body(request), "hm", "msgfolderroot");
 }
-
-FOLDER_STRUCT **BuildXMLFolderList(ghttp_request *request, FOLDER_STRUCT **folder_list, int *folder_cnt) {
-  return folderlist(ghttp_get_body(request), folder_list, folder_cnt);
-}
-
-char *build_propertyupdate_str(char *readmark) {
-  char *tmpxml;
-  int len = strlen(propupdate_read_start) + strlen(propupdate_read_end) + 2;
-  tmpxml = (char *) malloc(len);
-  strlcpy(tmpxml, propupdate_read_start, len);
-  strlcat(tmpxml, readmark, len);
-  strlcat(tmpxml, propupdate_read_end, len);
-
-  return tmpxml;
-}
